@@ -1,62 +1,46 @@
 
+
 /*
- * Copyright (C) 2000-2001 QuantLib Group
- *
- * This file is part of QuantLib.
- * QuantLib is a C++ open source library for financial quantitative
- * analysts and developers --- http://quantlib.sourceforge.net/
- *
- * QuantLib is free software and you are allowed to use, copy, modify, merge,
- * publish, distribute, and/or sell copies of it under the conditions stated
- * in the QuantLib License.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE. See the license for more details.
- *
- * You should have received a copy of the license along with this file;
- * if not, contact ferdinando@ametrano.net
- * The license is also available at http://quantlib.sourceforge.net/LICENSE.TXT
- *
- * The members of the QuantLib Group are listed in the Authors.txt file, also
- * available at http://quantlib.sourceforge.net/Authors.txt
-*/
+ Copyright (C) 2000, 2001, 2002 RiskMap srl
 
+ This file is part of QuantLib, a free-software/open-source library
+ for financial quantitative analysts and developers - http://quantlib.org/
+
+ QuantLib is free software: you can redistribute it and/or modify it under the
+ terms of the QuantLib license.  You should have received a copy of the
+ license along with this program; if not, please email ferdinando@ametrano.net
+ The license is also available online at http://quantlib.org/html/license.html
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ FOR A PARTICULAR PURPOSE.  See the license for more details.
+*/
 /*! \file array.hpp
     \brief 1-D array used in linear algebra.
 
-    $Id: array.hpp,v 1.1 2001/04/09 14:03:54 nando Exp $
-    $Source: /cvsroot/quantlib/QuantLib/Include/ql/array.hpp,v $
-    $Log: array.hpp,v $
-    Revision 1.1  2001/04/09 14:03:54  nando
-    all the *.hpp moved below the Include/ql level
-
-    Revision 1.3  2001/04/09 11:28:16  nando
-    updated copyright notice header and improved CVS tags
-
-    Revision 1.2  2001/04/06 18:46:19  nando
-    changed Authors, Contributors, Licence and copyright header
-
+    \fullpath
+    ql/%array.hpp
 */
 
+// $Id: array.hpp,v 1.10 2002/03/26 10:05:13 lballabio Exp $
 
 #ifndef quantlib_array_h
 #define quantlib_array_h
 
-#include "ql/qldefines.hpp"
-#include "ql/qlerrors.hpp"
+#include <ql/types.hpp>
+#include <ql/errors.hpp>
 #if QL_EXPRESSION_TEMPLATES_WORK
-#include "ql/expressiontemplates.hpp"
+    #include <ql/expressiontemplates.hpp>
 #endif
-#include <algorithm>
 #include <functional>
 #include <numeric>
-#include <iterator>
+#include <iostream>
 
 namespace QuantLib {
 
     //! 1-D array used in linear algebra.
-    /*! This class implements the concept of vector as used in linear algebra.
+    /*! This class implements the concept of vector as used in linear
+        algebra.
         As such, it is <b>not</b> meant to be used as a container -
         <tt>std::vector</tt> should be used instead.
     */
@@ -65,24 +49,24 @@ namespace QuantLib {
         //! \name Constructors, destructor, and assignment
         //@{
         //! creates the array with the given dimension
-        explicit Array(int size = 0);
+        explicit Array(Size size = 0);
         //! creates the array and fills it with <tt>value</tt>
-        Array(int size, double value);
-        /*! creates the array and fills it according to \f$ a_{0} = value,
-            a_{i}=a_{i-1}+increment \f$
+        Array(Size size, double value);
+        /*! \brief creates the array and fills it according to
+            \f$ a_{0} = value, a_{i}=a_{i-1}+increment \f$
         */
-        Array(int size, double value, double increment);
+        Array(Size size, double value, double increment);
         Array(const Array& from);
         #if QL_EXPRESSION_TEMPLATES_WORK
         template <class Iter> Array(const VectorialExpression<Iter>& e)
-        : pointer(0), n(0), bufferSize(0) { allocate(e.size()); copy(e); }
+        : pointer_(0), n_(0), bufferSize_(0) { allocate_(e.size()); copy_(e); }
         #endif
         ~Array();
         Array& operator=(const Array& from);
         #if QL_EXPRESSION_TEMPLATES_WORK
         template <class Iter> Array& operator=(
           const VectorialExpression<Iter>& e) {
-            resize(e.size()); copy(e); return *this;
+            resize_(e.size()); copy_(e); return *this;
         }
         #endif
         //@}
@@ -115,7 +99,7 @@ namespace QuantLib {
           const VectorialExpression<Iter>& e) {
             #ifdef QL_DEBUG
                 QL_REQUIRE(size() == e.size(),
-                    "adding arrays with different sizes")
+                    "adding arrays with different sizes");
             #endif
             iterator i = begin(), j = end();
             while (i != j) { *i += *e; ++i; ++e; }
@@ -125,7 +109,7 @@ namespace QuantLib {
           const VectorialExpression<Iter>& e) {
             #ifdef QL_DEBUG
                 QL_REQUIRE(size() == e.size(),
-                    "subtracting arrays with different sizes")
+                    "subtracting arrays with different sizes");
             #endif
             iterator i = begin(), j = end();
             while (i != j) { *i -= *e; ++i; ++e; }
@@ -135,7 +119,7 @@ namespace QuantLib {
           const VectorialExpression<Iter>& e) {
             #ifdef QL_DEBUG
                 QL_REQUIRE(size() == e.size(),
-                    "multiplying arrays with different sizes")
+                    "multiplying arrays with different sizes");
             #endif
             iterator i = begin(), j = end();
             while (i != j) { *i *= *e; ++i; ++e; }
@@ -145,7 +129,7 @@ namespace QuantLib {
           const VectorialExpression<Iter>& e) {
             #ifdef QL_DEBUG
                 QL_REQUIRE(size() == e.size(),
-                    "dividing arrays with different sizes")
+                    "dividing arrays with different sizes");
             #endif
             iterator i = begin(), j = end();
             while (i != j) { *i /= *e; ++i; ++e; }
@@ -156,14 +140,14 @@ namespace QuantLib {
         //! \name Element access
         //@{
         //! read-only
-        double operator[](int) const;
+        double operator[](Size) const;
         //! read-write
-        double& operator[](int);
+        double& operator[](Size);
         //@}
         //! \name Inspectors
         //@{
         //! dimension of the array
-        int size() const;
+        Size size() const;
         //@}
         typedef double* iterator;
         typedef const double* const_iterator;
@@ -182,13 +166,13 @@ namespace QuantLib {
         reverse_iterator rend();
         //@}
       private:
-        void allocate(int size);
-        void resize(int size);
-        void copy(const Array& from) {
+        void allocate_(Size size);
+        void resize_(Size size);
+        void copy_(const Array& from) {
             std::copy(from.begin(),from.end(),begin());
         }
         #if QL_EXPRESSION_TEMPLATES_WORK
-        template <class Iter> void copy(
+        template <class Iter> void copy_(
           const VectorialExpression<Iter>& e) {
             iterator i = begin(), j = end();
             while (i != j) {
@@ -197,8 +181,8 @@ namespace QuantLib {
             }
         }
         #endif
-        double* pointer;
-        int n, bufferSize;
+        double* pointer_;
+        Size n_, bufferSize_;
     };
 
     /*! \relates Array */
@@ -207,8 +191,8 @@ namespace QuantLib {
     // unary operators
 
     #if QL_EXPRESSION_TEMPLATES_WORK
-        // unary plus
 
+        // unary plus
         /*! \relates Array */
         VectorialExpression<
         UnaryVectorialExpression<Array::const_iterator,Plus> >
@@ -499,51 +483,51 @@ namespace QuantLib {
 
     // inline definitions
 
-    inline Array::Array(int size)
-    : pointer(0), n(0), bufferSize(0) {
+    inline Array::Array(Size size)
+    : pointer_(0), n_(0), bufferSize_(0) {
         if (size > 0)
-            allocate(size);
+            allocate_(size);
     }
 
-    inline Array::Array(int size, double value)
-    : pointer(0), n(0), bufferSize(0) {
+    inline Array::Array(Size size, double value)
+    : pointer_(0), n_(0), bufferSize_(0) {
         if (size > 0)
-            allocate(size);
+            allocate_(size);
         std::fill(begin(),end(),value);
     }
 
-    inline Array::Array(int size, double value, double increment)
-    : pointer(0), n(0), bufferSize(0) {
+    inline Array::Array(Size size, double value, double increment)
+    : pointer_(0), n_(0), bufferSize_(0) {
         if (size > 0)
-            allocate(size);
+            allocate_(size);
         for (iterator i=begin(); i!=end(); i++,value+=increment)
             *i = value;
     }
 
     inline Array::Array(const Array& from)
-    : pointer(0), n(0), bufferSize(0) {
-        allocate(from.size());
-        copy(from);
+    : pointer_(0), n_(0), bufferSize_(0) {
+        allocate_(from.size());
+        copy_(from);
     }
 
     inline Array::~Array() {
-        if (pointer != 0 && bufferSize != 0)
-            delete[] pointer;
-        pointer = 0;
-        n = bufferSize = 0;
+        if (pointer_ != 0 && bufferSize_ != 0)
+            delete[] pointer_;
+        pointer_ = 0;
+        n_ = bufferSize_ = 0;
     }
 
     inline Array& Array::operator=(const Array& from) {
         if (this != &from) {
-            resize(from.size());
-            copy(from);
+            resize_(from.size());
+            copy_(from);
         }
         return *this;
     }
 
     inline Array& Array::operator+=(const Array& v) {
         #ifdef QL_DEBUG
-            QL_REQUIRE(n == v.n,
+            QL_REQUIRE(n_ == v.n_,
                 "arrays with different sizes cannot be added");
         #endif
         std::transform(begin(),end(),v.begin(),begin(),std::plus<double>());
@@ -558,7 +542,7 @@ namespace QuantLib {
 
     inline Array& Array::operator-=(const Array& v) {
         #ifdef QL_DEBUG
-            QL_REQUIRE(n == v.n,
+            QL_REQUIRE(n_ == v.n_,
                 "arrays with different sizes cannot be subtracted");
         #endif
         std::transform(begin(),end(),v.begin(),begin(),
@@ -574,7 +558,7 @@ namespace QuantLib {
 
     inline Array& Array::operator*=(const Array& v) {
         #ifdef QL_DEBUG
-            QL_REQUIRE(n == v.n,
+            QL_REQUIRE(n_ == v.n_,
                 "arrays with different sizes cannot be multiplied");
         #endif
         std::transform(begin(),end(),v.begin(),begin(),
@@ -590,7 +574,7 @@ namespace QuantLib {
 
     inline Array& Array::operator/=(const Array& v) {
         #ifdef QL_DEBUG
-            QL_REQUIRE(n == v.n,
+            QL_REQUIRE(n_ == v.n_,
                 "arrays with different sizes cannot be divided");
         #endif
         std::transform(begin(),end(),v.begin(),begin(),
@@ -604,53 +588,53 @@ namespace QuantLib {
         return *this;
     }
 
-    inline double Array::operator[](int i) const {
+    inline double Array::operator[](Size i) const {
         #ifdef QL_DEBUG
-            QL_REQUIRE(i>=0 && i<n,
+            QL_REQUIRE(i<n_,
                 "array cannot be accessed out of range");
         #endif
-        return pointer[i];
+        return pointer_[i];
     }
 
-    inline double& Array::operator[](int i) {
+    inline double& Array::operator[](Size i) {
         #ifdef QL_DEBUG
-            QL_REQUIRE(i>=0 && i<n,
+            QL_REQUIRE(i<n_,
                 "array cannot be accessed out of range");
         #endif
-        return pointer[i];
+        return pointer_[i];
     }
 
-    inline int Array::size() const {
-        return n;
+    inline Size Array::size() const {
+        return n_;
     }
 
-    inline void Array::resize(int size) {
-        if (size != n) {
-            if (size <= bufferSize) {
-                n = size;
+    inline void Array::resize_(Size size) {
+        if (size != n_) {
+            if (size <= bufferSize_) {
+                n_ = size;
             } else {
                 Array temp(size);
                 std::copy(begin(),end(),temp.begin());
-                allocate(size);
-                copy(temp);
+                allocate_(size);
+                copy_(temp);
             }
         }
     }
 
     inline Array::const_iterator Array::begin() const {
-        return pointer;
+        return pointer_;
     }
 
     inline Array::iterator Array::begin() {
-        return pointer;
+        return pointer_;
     }
 
     inline Array::const_iterator Array::end() const {
-        return pointer+n;
+        return pointer_+n_;
     }
 
     inline Array::iterator Array::end() {
-        return pointer+n;
+        return pointer_+n_;
     }
 
     inline Array::const_reverse_iterator Array::rbegin() const {
@@ -669,30 +653,30 @@ namespace QuantLib {
         return reverse_iterator(begin());
     }
 
-    inline void Array::allocate(int size) {
-        if (pointer != 0 && bufferSize != 0)
-            delete[] pointer;
+    inline void Array::allocate_(Size size) {
+        if (pointer_ != 0 && bufferSize_ != 0)
+            delete[] pointer_;
         if (size <= 0) {
-            pointer = 0;
+            pointer_ = 0;
         } else {
-            n = size;
-            bufferSize = size+size/10+10;
+            n_ = size;
+            bufferSize_ = size+size/10+10;
             try {
-                pointer = new double[bufferSize];
+                pointer_ = new double[bufferSize_];
             }
             catch (...) {
-                pointer = 0;
+                pointer_ = 0;
             }
-            if (pointer == 0) {
-                n = bufferSize = size;
+            if (pointer_ == 0) {
+                n_ = bufferSize_ = size;
                 try {
-                    pointer = new double[bufferSize];
+                    pointer_ = new double[bufferSize_];
                 }
                 catch (...) {
-                    pointer = 0;
+                    pointer_ = 0;
                 }
-                if (pointer == 0) {
-                    n = bufferSize = 0;
+                if (pointer_ == 0) {
+                    n_ = bufferSize_ = 0;
                     throw OutOfMemoryError("Array");
                 }
             }
@@ -910,7 +894,7 @@ namespace QuantLib {
         operator-(const Array& v1, const Array& v2) {
             #ifdef QL_DEBUG
                 QL_REQUIRE(v1.size() == v2.size(),
-                    "adding arrays with different sizes");
+                    "subtracting arrays with different sizes");
             #endif
             typedef BinaryVectorialExpression<
             Array::const_iterator,Array::const_iterator,Subtract>
@@ -944,7 +928,7 @@ namespace QuantLib {
         operator-(const Array& v1, const VectorialExpression<Iter2>& e2) {
             #ifdef QL_DEBUG
                 QL_REQUIRE(v1.size() == e2.size(),
-                    "adding arrays with different sizes");
+                    "subtracting arrays with different sizes");
             #endif
             typedef BinaryVectorialExpression<
                 Array::const_iterator,VectorialExpression<Iter2>,Subtract>
@@ -960,7 +944,7 @@ namespace QuantLib {
         operator-(const VectorialExpression<Iter1>& e1, const Array& v2) {
             #ifdef QL_DEBUG
                 QL_REQUIRE(e1.size() == v2.size(),
-                    "adding arrays with different sizes");
+                    "subtracting arrays with different sizes");
             #endif
             typedef BinaryVectorialExpression<
                 VectorialExpression<Iter1>,Array::const_iterator,Subtract>
@@ -977,7 +961,7 @@ namespace QuantLib {
             const VectorialExpression<Iter2>& e2) {
             #ifdef QL_DEBUG
                 QL_REQUIRE(e1.size() == e2.size(),
-                    "adding arrays with different sizes");
+                    "subtracting arrays with different sizes");
             #endif
             typedef BinaryVectorialExpression<
                 VectorialExpression<Iter1>,VectorialExpression<Iter2>,Subtract>
@@ -1045,7 +1029,7 @@ namespace QuantLib {
         operator*(const Array& v1, const Array& v2) {
             #ifdef QL_DEBUG
                 QL_REQUIRE(v1.size() == v2.size(),
-                    "adding arrays with different sizes");
+                    "multiplying arrays with different sizes");
             #endif
             typedef BinaryVectorialExpression<
             Array::const_iterator,Array::const_iterator,Multiply>
@@ -1079,7 +1063,7 @@ namespace QuantLib {
         operator*(const Array& v1, const VectorialExpression<Iter2>& e2) {
             #ifdef QL_DEBUG
                 QL_REQUIRE(v1.size() == e2.size(),
-                    "adding arrays with different sizes");
+                    "multiplying arrays with different sizes");
             #endif
             typedef BinaryVectorialExpression<
                 Array::const_iterator,VectorialExpression<Iter2>,Multiply>
@@ -1095,7 +1079,7 @@ namespace QuantLib {
         operator*(const VectorialExpression<Iter1>& e1, const Array& v2) {
             #ifdef QL_DEBUG
                 QL_REQUIRE(e1.size() == v2.size(),
-                    "adding arrays with different sizes");
+                    "multiplying arrays with different sizes");
             #endif
             typedef BinaryVectorialExpression<
                 VectorialExpression<Iter1>,Array::const_iterator,Multiply>
@@ -1112,7 +1096,7 @@ namespace QuantLib {
             const VectorialExpression<Iter2>& e2) {
             #ifdef QL_DEBUG
                 QL_REQUIRE(e1.size() == e2.size(),
-                    "adding arrays with different sizes");
+                    "multiplying arrays with different sizes");
             #endif
             typedef BinaryVectorialExpression<
                 VectorialExpression<Iter1>,VectorialExpression<Iter2>,Multiply>
@@ -1180,7 +1164,7 @@ namespace QuantLib {
         operator/(const Array& v1, const Array& v2) {
             #ifdef QL_DEBUG
                 QL_REQUIRE(v1.size() == v2.size(),
-                    "adding arrays with different sizes");
+                    "dividing arrays with different sizes");
             #endif
             typedef BinaryVectorialExpression<
                 Array::const_iterator,Array::const_iterator,Divide>
@@ -1214,7 +1198,7 @@ namespace QuantLib {
         operator/(const Array& v1, const VectorialExpression<Iter2>& e2) {
             #ifdef QL_DEBUG
                 QL_REQUIRE(v1.size() == e2.size(),
-                    "adding arrays with different sizes");
+                    "dividing arrays with different sizes");
             #endif
             typedef BinaryVectorialExpression<
                 Array::const_iterator,VectorialExpression<Iter2>,Divide>
@@ -1230,7 +1214,7 @@ namespace QuantLib {
         operator/(const VectorialExpression<Iter1>& e1, const Array& v2) {
             #ifdef QL_DEBUG
                     QL_REQUIRE(e1.size() == v2.size(),
-                        "adding arrays with different sizes");
+                        "dividing arrays with different sizes");
             #endif
             typedef BinaryVectorialExpression<
                 VectorialExpression<Iter1>,Array::const_iterator,Divide>
@@ -1247,7 +1231,7 @@ namespace QuantLib {
             const VectorialExpression<Iter2>& e2) {
             #ifdef QL_DEBUG
                 QL_REQUIRE(e1.size() == e2.size(),
-                    "adding arrays with different sizes");
+                    "dividing arrays with different sizes");
             #endif
             typedef BinaryVectorialExpression<
                 VectorialExpression<Iter1>,VectorialExpression<Iter2>,Divide>
